#-----------------------------------------------------------------------------
require 'sketchup.rb'
#-----------------------------------------------------------------------------


module JBB_LayersPanel
	
	model = Sketchup.active_model
	layers = model.layers
	ents = model.entities
	defs = model.definitions
	
	
	### LAYER DELETE METHOD ### ------------------------------------------------------
	#Simple modification of TIG's snippet delete-layer.rb
	#Basically, move or delete layer content, then creates an entry for every layer except one to delete, then purge layers, then delete entries
	
	def self.deleteLayer(layer, delete_geometry=false)
		model=Sketchup.active_model
		ents=model.entities; defs=model.definitions; layers=model.layers
		model.start_operation ("Delete layer")
		if delete_geometry
			allents=[]
			ents.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
			defs.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
			allents.each{|e|e.erase! if e.valid?}
		else ### move geom to Layer0 etc
			ents.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}
			defs.each{|d|d.entities.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}}
		end#if
		group=ents.add_group();gents=group.entities ### temporarily use other layers
		temp=gents.add_group()
		temp.layer=nil
		(layers.to_a-[layer]).each{|layer|tc=temp.copy;tc.layer=layer}
		model.active_layer=nil if model.active_layer==layer ### ensure layer is not current layer
		layers.purge_unused ### purge layer from browser
		group.erase! ### erase! the temporary layer user, use set as was.
		model.commit_operation 
	end#def
  
  
  
	### WEBDIALOGS & CALLBACKS ### ------------------------------------------------------
	
	# Create the WebDialog instance
	# Global variable, otherwise it doesn't work...
	$jbb_layersPanel_dialog = UI::WebDialog.new("Layers Panel", false, "LayersPanel", 300, 300, 300, 200, true)
	$jbb_layersPanel_dialog.min_width = 270
	html_path = File.dirname( __FILE__ ) + "/Layers Panel/layers Panel.html"
	$jbb_layersPanel_dialog.set_file(html_path)

	$jbb_layersPanel_dialog.add_action_callback("getModelLayers") do  |wdl, action|
		layerID = 0
		layersLength = layers.length
		while layerID < layersLength
			layer = layers[layerID]
			layerIdForJS = layerID.to_s
			addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{layerIdForJS}');"
			wdl.execute_script(addLayerFromRuby)
			if layer.visible?
				showLayerFromRuby = "showLayerFromRuby('#{layer.name}');"
				wdl.execute_script(showLayerFromRuby)
			else
				showLayerFromRuby = "hideLayerFromRuby('#{layer.name}');"
				wdl.execute_script(showLayerFromRuby)
			end#if
			layerID = layerID + 1
		end#while
	end#callback getModelLayers

	$jbb_layersPanel_dialog.add_action_callback("getActiveLayer") do  |wdl, action|
		activeLayer = model.active_layer.name
		setActiveLayerFromRuby = "setActiveLayerFromRuby('#{activeLayer}');"
		wdl.execute_script(setActiveLayerFromRuby)
	end#callback getModelLayers

	$jbb_layersPanel_dialog.add_action_callback("setActiveLayerFromJS") do  |wdl, layerName|
		newActiveLayer = layers[layerName]
		model.active_layer = newActiveLayer
	end#callback getModelLayers

	$jbb_layersPanel_dialog.add_action_callback("addLayerFromJS") do |wdl, addedLayerName|
		layers.add addedLayerName
	end#callback addLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("renameLayerFromJS") do |wdl, LayerNameS|
		hashStringLayerNames = eval( LayerNameS.inspect.gsub(':','=>') ) #Convert Json string to Hash string
		hashLayerNames = eval(hashStringLayerNames) #Convert hash string to proper hash object
		oldLayerName = hashLayerNames['oldLayerName']
		newLayerName = hashLayerNames['newLayerName']
		layers[oldLayerName].name = newLayerName
	end#callback renameLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("checkLayerForContent") do |wdl, layerName|
		layer = layers[layerName]
		layerHasContent = false
		
		allents=[]
		ents.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
		defs.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
		if allents != [] #Layer has content
			layerHasContent = true
		end#if
		
		checkLayerForContent = "checkLayerForContent('#{layerName}', '#{layerHasContent}');"
		$jbb_layersPanel_dialog.execute_script(checkLayerForContent)
	end#callback deleteLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("deleteLayerFromJS") do |wdl, layerName|
		self.deleteLayer(layers[layerName])
	end#callback deleteLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("deleteLayer&GeomFromJS") do |wdl, layerName|
		self.deleteLayer(layers[layerName], true)
	end#callback deleteLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("hideLayerFromJS") do |wdl, layerName|
		layers[layerName].visible = false
	end#callback hideLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("showLayerFromJS") do |wdl, layerName|
		layers[layerName].visible = true
	end#callback showLayerFromJS

	$jbb_layersPanel_dialog.add_action_callback("purgeLayersFromJS") do |wdl, act|
		layers.purge_unused
	end#callback showLayerFromJS
	
  
  
	### LAYERSOBSERVER ### ------------------------------------------------------
	
	# Layers observer
	class JBB_LP_layersObserver < Sketchup::LayersObserver
	
		def onLayerAdded(layers, layer)
			addLayerFromRuby = "addLayerFromRuby('#{layer.name}');"
			$jbb_layersPanel_dialog.execute_script(addLayerFromRuby)
		end#onLayerAdded
		
		def onLayerRemoved(layers, layer)
			deleteLayerFromRuby = "deleteLayerFromRuby('#{layer.name}');"
			$jbb_layersPanel_dialog.execute_script(deleteLayerFromRuby)
		end#onLayerRemoved
		
		
		def onCurrentLayerChanged(layers, layer)
			$jbb_layersPanel_dialog.execute_script('getActiveLayer();')
		end#onLayerRemoved
		
	end#JBB_LP_layersObserver
	
	# Attach the observer.
	layers.add_observer(JBB_LP_layersObserver.new)
  
  
  
	### MENU & TOOLBARS ### ------------------------------------------------------
	
	def self.toggle_layerspanel_dlg
		if $jbb_layersPanel_dialog.visible?
			$jbb_layersPanel_dialog.close
			false
		else
			$jbb_layersPanel_dialog.show()
			true
		end#if
	end#def
	
	def self.layerspanel_dlg_validation_proc
		if $jbb_layersPanel_dialog && $jbb_layersPanel_dialog.visible?
			MF_CHECKED
		else
			MF_UNCHECKED
		end#if
	end#def
  
	unless file_loaded?( __FILE__ )
		# Commands
		cmd = UI::Command.new( 'Layers Panel' ) { self.toggle_layerspanel_dlg }
		cmd.status_bar_text = 'Show or hide the Layers Panel.'
		cmd.small_icon = "Layers Panel/lp_16.png"
		cmd.large_icon = "Layers Panel/lp_24.png"
		cmd.tooltip = 'Layers Panel'
		cmd.set_validation_proc { self.layerspanel_dlg_validation_proc }
		cmd_toggle_layerspanel_dlg = cmd

		window_menu = UI.menu("Window")
		window_menu.add_item( cmd_toggle_layerspanel_dlg )
		
		layerspanel_tb = UI::Toolbar.new "Layers Panel"
		layerspanel_tb.add_item cmd_toggle_layerspanel_dlg
		layerspanel_tb.show
	end 
	
end#module

#-----------------------------------------------------------------------------
file_loaded( File.basename(__FILE__) )
